#include <QToolBox>
#include <QRadioButton>

#include "componentchoosermultimedia.h"
#include <kbuildsycocaprogressdialog.h>
#include <kdebug.h>
#include <kmimetypetrader.h>
#include <kglobalsettings.h>
#include <kconfiggroup.h>
#include <KServiceTypeTrader>
#include <KMimeType>

CfgMultimedia::CfgMultimedia(QWidget *parent)
    : QWidget(parent), CfgPlugin()
{
    bool is_kde = 0 == qstrnicmp("true", qgetenv("KDE_FULL_SESSION"), 4);

    QVBoxLayout *lay = new QVBoxLayout(this);
    QToolBox *toolbox = new QToolBox(this);

    vplayers_page = new QWidget(toolbox);
    v_lay = new QVBoxLayout(vplayers_page);
    toolbox->addItem(vplayers_page, KIcon("video-x-generic"), i18n("Video Player"));
    aplayers_page = new QWidget(toolbox);
    a_lay = new QVBoxLayout(aplayers_page);
    toolbox->addItem(aplayers_page, KIcon("audio-x-generic"), i18n("Audio Player"));
    iplayers_page = new QWidget(toolbox);
    i_lay = new QVBoxLayout(iplayers_page);
    toolbox->addItem(iplayers_page, KIcon("image-x-generic"), i18n("Image Viewer"));
    lay->addWidget(toolbox);

    const KService::List services  = KServiceTypeTrader::self()->query(QLatin1String("Application"));
    Q_FOREACH (KService::Ptr service_ptr, services)
    {
	KService *service(service_ptr.data());
	if( is_kde && service->noDisplay() )
	    continue;
	if( service->categories().contains("Player") ) {
	    if( service->categories().contains("AudioVideo") || service->categories().contains("Video") ) {
		Q_FOREACH(QString mimetype, service->mimeTypes())
		{
		    if( mimetype.startsWith( QString::fromLatin1("video/") ) 
			&& !services_video.contains(service_ptr) ) {
			services_video << service_ptr;
			break;
		    }
		}
	    }
	    if( service->categories().contains("Audio") ) {
		Q_FOREACH(QString mimetype, service->mimeTypes())
		{
		    if(  mimetype.startsWith( QString::fromLatin1("audio/") )
			&& !services_audio.contains(service_ptr) ) {
			services_audio << service_ptr;
			break;
		    }
		}
	    }
	} else if( service->categories().contains("Viewer") && service->categories().contains("Graphics") ) {
	    Q_FOREACH(QString mimetype, service->mimeTypes())
	    {
		if( !services_image.contains(service_ptr) ) {
		    services_image << service_ptr;
		    break;
		}
	    }
	}
    }
}

CfgMultimedia::~CfgMultimedia() {
}

void CfgMultimedia::configChanged()
{
    emit changed(true);
}

void CfgMultimedia::defaults()
{
    load(0);
}

void CfgMultimedia::load(KConfig *) {
    QLayoutItem *child_item;
    qDeleteAll(vplayers_page->findChildren<QRadioButton*>());
    while( (child_item = v_lay->takeAt(0)) != 0 ) {
	delete child_item;
    }
    Q_FOREACH(KService::Ptr service, services_video)
    {
	QString text(service->name());
	if( !service->genericName().isEmpty() ) {
	    text.append(" (");
	    text.append(service->genericName());
	    text.append(")");
	}
	QRadioButton* button = new QRadioButton(text, vplayers_page);
	connect(button,SIGNAL(toggled(bool)),this,SLOT(configChanged()));
	button->setProperty("storageId", service->storageId());
	if( !service->comment().isEmpty() ) {
	    button->setToolTip(service->comment());
	} else {
	    button->setToolTip(service->exec());
	}
	v_lay->addWidget(button);
    }
    v_lay->addStretch();

    qDeleteAll(aplayers_page->findChildren<QRadioButton*>());
    while( (child_item = a_lay->takeAt(0)) != 0 ) {
	delete child_item;
    }
    Q_FOREACH(KService::Ptr service, services_audio)
    {
	QString text(service->name());
	if( !service->genericName().isEmpty() ) {
	    text.append(" (");
	    text.append(service->genericName());
	    text.append(")");
	}
	QRadioButton* button = new QRadioButton(text, aplayers_page);
	connect(button,SIGNAL(toggled(bool)),this,SLOT(configChanged()));
	button->setProperty("storageId", service->storageId());
	if( !service->comment().isEmpty() ) {
	    button->setToolTip(service->comment());
	} else {
	    button->setToolTip(service->exec());
	}
	a_lay->addWidget(button);
    }
    a_lay->addStretch();

    qDeleteAll(iplayers_page->findChildren<QRadioButton*>());
    while( (child_item = i_lay->takeAt(0)) != 0 ) {
	delete child_item;
    }
    Q_FOREACH(KService::Ptr service, services_image)
    {
	QString text(service->name());
	if( !service->genericName().isEmpty() ) {
	    text.append(" (");
	    text.append(service->genericName());
	    text.append(")");
	}
	QRadioButton* button = new QRadioButton(text, iplayers_page);
	connect(button,SIGNAL(toggled(bool)),this,SLOT(configChanged()));
	button->setProperty("storageId", service->storageId());
	if( !service->comment().isEmpty() ) {
	    button->setToolTip(service->comment());
	} else {
	    button->setToolTip(service->exec());
	}
	i_lay->addWidget(button);
    }
    i_lay->addStretch();

    emit changed(false);
}

void CfgMultimedia::save(KConfig *)
{
    QStringList mimetypes_video;
    QStringList mimetypes_audio;
    QStringList mimetypes_image;

// get selected items
    Q_FOREACH(QRadioButton* button, qFindChildren<QRadioButton*>(vplayers_page)) {
        if (button->isChecked()) {
            video_player_id = button->property("storageId").toString();
            break;
        }
    }
    Q_FOREACH(QRadioButton* button, qFindChildren<QRadioButton*>(aplayers_page)) {
        if (button->isChecked()) {
            audio_player_id = button->property("storageId").toString();
            break;
        }
    }
    Q_FOREACH(QRadioButton* button, qFindChildren<QRadioButton*>(iplayers_page)) {
        if (button->isChecked()) {
            image_player_id = button->property("storageId").toString();
            break;
        }
    }

// collect mimetypes
	KMimeType::List kmimetypes = KMimeType::allMimeTypes();
	KMimeType::List::const_iterator it_mime(kmimetypes.constBegin());
	for (; it_mime != kmimetypes.constEnd(); it_mime++)
	{
	    const QString mimetype = (*it_mime)->name();
	    if( mimetype.startsWith(QString::fromLatin1("video/")) ) {
		mimetypes_video.append(mimetype);
	    } else if( mimetype.startsWith(QString::fromLatin1("audio/")) ) {
		mimetypes_audio.append(mimetype);
	    } else if( mimetype.startsWith(QString::fromLatin1("image/")) ) {
		mimetypes_image.append(mimetype);
	    }
	}

// save
        KSharedConfig::Ptr profile = KSharedConfig::openConfig("mimeapps.list", KConfig::NoGlobals, "xdgdata-apps");
        if ( profile->isConfigWritable(true) )
        {
	    KConfigGroup addedApps(profile, "Added Associations");
	    if (!video_player_id.isEmpty()) {
		Q_FOREACH(QString mimetype, mimetypes_video)
		{
		    QStringList userApps = addedApps.readXdgListEntry(mimetype);
		    userApps.removeAll(video_player_id);
		    userApps.prepend(video_player_id);
		    addedApps.writeXdgListEntry(mimetype, userApps);
		}
	    }
	    if (!audio_player_id.isEmpty()) {
		Q_FOREACH(QString mimetype, mimetypes_audio)
		{
		    QStringList userApps = addedApps.readXdgListEntry(mimetype);
		    userApps.removeAll(audio_player_id);
		    userApps.prepend(audio_player_id);
		    addedApps.writeXdgListEntry(mimetype, userApps);
		}
	    }
	    if (!image_player_id.isEmpty()) {
		Q_FOREACH(QString mimetype, mimetypes_image)
		{
		    QStringList userApps = addedApps.readXdgListEntry(mimetype);
		    userApps.removeAll(image_player_id);
		    userApps.prepend(image_player_id);
		    addedApps.writeXdgListEntry(mimetype, userApps);
		}
	    }
	    profile->sync();
	    if( !video_player_id.isEmpty() || !audio_player_id.isEmpty() || !image_player_id.isEmpty() )
		KBuildSycocaProgressDialog::rebuildKSycoca(this);
        }

    emit changed(false);
}
